/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AdminDestinationMBean.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;

/**
 * MBean Interface for administer JMS destination and physic queue
 * 
 * @author Kien THAI
 * 
 */
public interface AdminDestinationMBean {
	
	/**
	 * 
	 * @return		true if this destination is a topic
	 * 				false if this destination is a queue
	 */
	public boolean getIsTopic();
	
	/**
	 * Get JMS property
	 * 
	 * @param Property
	 *            name
	 * @return Property value
	 * @throws Exception
	 */
	public String getProperty(String propertyName) throws Exception;

	/**
	 * Set JMS property
	 * 
	 * @param Property
	 *            name
	 * @param Property
	 *            value
	 * @throws Exception
	 */
	public void setProperty(String propertyName, String propertyValue)
			throws Exception;

	/**
	 * Get queue depth
	 * 
	 * @return Queue Depth
	 * @throws Exception
	 */
	public int getCurrentDepth() throws Exception;

	/**
	 * Clear queue messages
	 * 
	 * @throws Exception
	 */
	public void clearQueue() throws Exception;

	/**
	 * Get queue input count
	 * 
	 * @return
	 * @throws Exception
	 */
	public int getInputCount() throws Exception;

	/**
	 * Get queue output count
	 * 
	 * @return
	 * @throws Exception
	 */
	public int getOutputCount() throws Exception;

	
	public boolean getDepthHighEvent() throws Exception;
	
	/**
	 * Enable or disable queue depth high event
	 * 
	 * @param Enable
	 *            or disable
	 * @throws Exception
	 */
	public void setDepthHighEvent(boolean enable) throws Exception;

	public int getMaxQueueDepth() throws Exception;

	/**
	 * Set queue depth high limit
	 * 
	 * @param queue
	 *            depth high limit
	 * @throws Exception
	 */
	public void setDepthHighLimit(int depthHighLimit) throws Exception;

	/**
	 * Get queue depth high limit
	 * 
	 * @return
	 * @throws Exception
	 */
	public int getDepthHighLimit() throws Exception;

	/**
	 * Set type of queue service interval event
	 * 
	 * @param Type
	 *            of service interval event
	 * @throws Exception
	 */
	public void setServiceIntervalEvent(int type) throws Exception;

	/**
	 * Get type of queue service interval event
	 * 
	 * @return Type of service interval event
	 * @throws Exception
	 */
	public int getServiceIntervalEvent() throws Exception;

	/**
	 * Set queue service interval
	 * 
	 * @param Queue
	 *            service interval
	 * @throws Exception
	 */
	public void setServiceInterval(int interval) throws Exception;

	/**
	 * Get queue service interval
	 * 
	 * @return Queue service interval
	 * @throws Exception
	 */
	public int getServiceInterval() throws Exception;
	
	public boolean getDefPersistence() throws Exception;
	
	/**
	 * Enable or disable message persistence on this queue
	 * 
	 * @param Enable
	 *            or disable
	 * @throws Exception
	 */
	public void setDefPersistence(boolean enable) throws Exception;

}
