/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JmsAdminMBean.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;


import javax.resource.ResourceException;

public interface JmsAdminMBean {

	public String getTransportType();

	public String getHostName();

	public String getPort();

	public String getChannel();

	public String getQueueManager();

	public void setTransportType(String transportType) throws Exception;

	public void setHostName(String hostName) throws Exception;

	public void setPort(String port) throws Exception;

	public void setChannel(String channel) throws Exception;

	public void setQueueManager(String queueManager) throws Exception;

	/**
	 * Retrieve Connection Factory Property
	 * 
	 * @param propertyName
	 *            Property name
	 * @return A String represent property value
	 * @throws Exception
	 *             If property can not be retrieved
	 */
	public String getCfProperty(String propertyName) throws Exception;

	/**
	 * Set Connection Factory Property
	 * 
	 * @param propertyName
	 *            Property name
	 * @param propertyValue
	 *            A String represent property value
	 * @throws Exception
	 *             If property can not be set
	 */
	public void setCfProperty(String propertyName, String propertyValue)
			throws Exception;

	/**
	 * Retrieve Queue Connection Factory Property
	 * 
	 * @param propertyName
	 *            Property name
	 * @return A String represent property value
	 * @throws Exception
	 *             If property can not be retrieved
	 */
	public String getQcfProperty(String propertyName) throws Exception;

	/**
	 * Set Queue Connection Factory Property
	 * 
	 * @param propertyName
	 *            Property name
	 * @param propertyValue
	 *            A String represent property value
	 * @throws Exception
	 *             If property can not be set
	 */
	public void setQcfProperty(String propertyName, String propertyValue)
			throws Exception;

	/**
	 * Retrieve Topic Connection Factory Property
	 * 
	 * @param propertyName
	 *            Property name
	 * @return A String represent property value
	 * @throws Exception
	 *             If property can not be retrieved
	 */
	public String getTcfProperty(String propertyName) throws Exception;

	/**
	 * Set Topic Connection Factory Property
	 * 
	 * @param propertyName
	 *            Property name
	 * @param propertyValue
	 *            A String represent property value
	 * @throws Exception
	 *             If property can not be set
	 */
	public void setTcfProperty(String propertyName, String propertyValue)
			throws Exception;

	/**
	 * Create or get destination from JNDI Context of WS MQ
	 * 
	 * @param isTopic
	 *            Destination is a topic or not
	 * @param jndiName
	 *            JNDI name of the destination
	 * @param properties
	 *            Properties of the destination
	 * 
	 * @throws ResourceException
	 *             if destination can not be retrieved
	 */
	public void createDestination(boolean isTopic, String jndiName,
			String properties) throws ResourceException;

	/**
	 * Delete a destination
	 * 
	 * @param jndiName
	 *            JNDI name of the destination
	 * @param deleteJmsDest
	 *            JMS Destination should be deleted or not
	 * @throws ResourceException
	 *             If the destination can not be deleted
	 */
	public void deleteDestination(String jndiName, boolean deletePhysicDest)
			throws ResourceException;

	/**
	 * Set performance event enabled or disabled
	 * 
	 * @param enable
	 * @throws Exception
	 */
	public void setPerformanceEventEnabled(boolean enable) throws Exception;

	public boolean getPerformanceEventEnabled() throws Exception;
	
	public void saveDestinationConfig() throws Exception;
	
	public void loadDestinationConfig() throws Exception;
	
	public String[] listDestinationObjectNames();
	
	public String[] listConsumerObjectNames();
	
	public void restartAdminQm();
	
	public String[] listPropertyNames(String jmsObject);
}
