/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: XmlConfig.java 10252 2007-04-23 11:27:02Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.admin;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;

import com.bull.jonasmqconnector.util.ExceptionUtils;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

//Should use DefaultHandler?
public class XmlConfig extends DefaultHandler {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	private String fileName = null;

	private JmsAdmin jmsAdmin = null;

	private Attributes destinationAttrs = null;

	private Hashtable destinationProperties = null;

	public XmlConfig(String MQDestinationConfigFile, JmsAdmin admin)
			throws Exception {
		logger.debug("XmlConfig#XmlConfig(" + MQDestinationConfigFile + ")");

		jmsAdmin = admin;
		
		try {
			
			fileName = Thread.currentThread().getContextClassLoader()
			.getResource(MQDestinationConfigFile).getPath();
			logger.info("XmlConfig#XmlConfig config file path="+fileName);
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not handle XML Config file",
					e);
		}
	}

	public void loadConfig() throws Exception {
		logger.debug("XmlConfig#loadConfig()");

		// TODO Use the default (non-validating) parser?
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			factory.setNamespaceAware(false);
			SAXParser saxParser = factory.newSAXParser();
			
			FileInputStream is = new FileInputStream(fileName);
			
			saxParser.parse(is, this);
			is.close();
		} catch (Exception e) {
			
			throw ExceptionUtils.newException("Can not handle XML Config file",
					e);
		}
	}

	public void saveConfig() throws Exception {
		logger.debug("XmlConfig#saveConfig()");
		Object[] destinationArray = jmsAdmin.getAdminDestinations().values()
				.toArray();

		AdminDestination dest = null;
		Hashtable properties = null;

		String propertyName = null;
		String propertyValue = null;
		String type = null;

		Element destinationElement = null;
		Element propertyElement = null;

		DocumentBuilderFactory domFactory = null;
		DocumentBuilder domBuilder = null;
		try {
			domFactory = DocumentBuilderFactory.newInstance();
			domBuilder = domFactory.newDocumentBuilder();
		} catch (Exception e) {
			throw ExceptionUtils.newException("Can not initialize DOM builder",
					e);
		}

		Document xmlDoc = domBuilder.newDocument();

		Element rootElement = xmlDoc.createElement("jonasmq");
		xmlDoc.appendChild(rootElement);

		Element destinationsElement = xmlDoc.createElement("destinations");
		rootElement.appendChild(destinationsElement);

		for (int i = 0; i < destinationArray.length; i++) {
			try {
				dest = (AdminDestination) destinationArray[i];
				properties = dest.getPropertiesTable();

				destinationElement = xmlDoc.createElement("destination");
				destinationElement.setAttribute("name", dest.getJndiName());

				type = (dest.getIsTopic()) ? "Topic" : "Queue";
				destinationElement.setAttribute("type", type);

				for (Enumeration e = properties.keys(); e.hasMoreElements();) {

					propertyName = e.nextElement().toString();

					propertyValue = properties.get(propertyName).toString();
					propertyElement = xmlDoc.createElement("property");
					propertyElement.setAttribute("name", propertyName);
					propertyElement.setAttribute("value", propertyValue);
					destinationElement.appendChild(propertyElement);
				}

				destinationsElement.appendChild(destinationElement);
			} catch (Exception e) {
				
				logger.error("Can not add " + dest.getJndiName()
						+ " destination on XML doc", e);
			}
		}
		
		try {		
			// Save the document to the disk file
			TransformerFactory tranFactory = TransformerFactory.newInstance();
			Transformer tran = tranFactory.newTransformer();
			
			Source source = new DOMSource(xmlDoc);
			
			FileOutputStream os = new FileOutputStream(fileName);
			
			Result result = new StreamResult(os);
			
			//tran.setOutputProperty(OutputKeys.METHOD, "xml");
			tran.setOutputProperty(OutputKeys.MEDIA_TYPE,"text/xml");
			tran.setOutputProperty(OutputKeys.INDENT, "yes");
			
			tran.transform(source, result);
			
			os.close();
		} catch (Exception e) {
			logger.error("Can not save config file",e);
		}
	}

	/***************************************************************************
	 * SAX DocumentHandler methods
	 **************************************************************************/

	public void startDocument() throws SAXException {
		logger.debug("XmlConfig#startDocument()");
	}

	public void startElement(String namespaceURI, String sName, String qName,
			Attributes attrs) throws SAXException {
		logger.debug("XmlConfig#startElement(" + namespaceURI + "," + sName
				+ "," + qName + "," + attrs + ")");
		if (qName != null && !qName.equalsIgnoreCase("property")) {
			destinationProperties = new Hashtable();
			destinationAttrs = new AttributesImpl(attrs);
		} else if (attrs != null) {
			String propName = null, propValue = null;
			for (int i = 0; i < attrs.getLength(); i++) {
				if (attrs.getQName(i) != null
						&& attrs.getQName(i).equalsIgnoreCase("name")) {
					propName = attrs.getValue(i);
				}
				if (attrs.getQName(i) != null
						&& attrs.getQName(i).equalsIgnoreCase("value")) {
					propValue = attrs.getValue(i);
				}
			}
			if (propName != null && propValue != null) {
				destinationProperties.put(propName, propValue);
			}
		}
	}

	public void endElement(String namespaceURI, String sName, String qName)
			throws SAXException {
		logger.debug("XmlConfig#endElement(" + namespaceURI + "," + sName + ","
				+ qName + ")");
		try {
			if (qName.equalsIgnoreCase("destination")) {
				endDestinationElement(namespaceURI, sName, qName);
			}
		} catch (Exception ex) {
			throw new SAXException(ex);
		}
	}

	public void endDocument() throws SAXException {
		logger.debug("XmlConfig#endDocument()");
		destinationProperties = null;
		destinationAttrs = null;
	}

	/***************************************************************************
	 * PRIVATE METHODS
	 **************************************************************************/

	private void endDestinationElement(String namespaceURI, String sName,
			String qName) throws Exception {
		logger.debug("XmlConfig#endDestinationElement(" + namespaceURI + "," 
				+ sName + "," + qName + ")");
		String destName = null;
		String destType = null;
		for (int i = 0; i < destinationAttrs.getLength(); i++) {
			if (destinationAttrs.getQName(i) != null
					&& destinationAttrs.getQName(i).equalsIgnoreCase("name")) {
				destName = destinationAttrs.getValue(i);
			}
			if (destinationAttrs.getQName(i) != null
					&& destinationAttrs.getQName(i).equalsIgnoreCase("type")) {
				destType = destinationAttrs.getValue(i);
			}
		}
		if (destType.equalsIgnoreCase("Queue"))
			jmsAdmin.createDestination(false, destName, destinationProperties);
		else if (destType.equalsIgnoreCase("Topic"))
			jmsAdmin.createDestination(true, destName, destinationProperties);
		else
			logger.warn("Can not identify destination type: " + destType);
	}
}