/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ExceptionListenerImpl.java 10253 2007-04-23 11:27:55Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.inbound;

import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.resource.ResourceException;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * A helper class used to reestablish the connection if exception is generated.
 * 
 * @author Kien THAI
 * 
 */
public class ExceptionListenerImpl implements ExceptionListener {
	
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();
	
	private InboundConsumer consumer;
	
	// TODO To parameterize these variables
	int maxReconnect;
	long interval;
	
	
	public ExceptionListenerImpl(InboundConsumer consumer) {
		this.consumer = consumer;
		this.maxReconnect = consumer.getSpec().getMaxReconnect();
		this.interval = consumer.getSpec().getReconnectInterval();
	}
	
	public synchronized void onException(JMSException connectionException) {
		logger.warn("ExceptionListenerImpl#onException(" + connectionException
				+ ")");
		
		boolean cont = true;
		int attempt = 0;
		
		while (cont && (attempt < maxReconnect))
		{
			attempt++;
			
			logger.warn("Reconnect Attempt : " + attempt + " of " + maxReconnect +
					" -> Sleep " + interval + " miliseconds");
			
			try {
				Thread.sleep(interval);
			} catch (InterruptedException  ie) {
				logger.warn("Thread interrupted: "+ie);
			}

			// TODO To test this
//			if (!consumer.isStop()) {
//				try {
//					logger.info("ExceptionListenerImpl#onException "+
//					"try to stop consumer...");
//					consumer.stop();
//				} catch (ResourceException re) {
//					logger.warn("ExceptionListenerImpl#onException "+
//							"consumer.stop", re);
//					continue;
//				}
//			}
			
			// TODO To test this
			// Try to reconnect
			try {
				logger.info("ExceptionListenerImpl#onException "+
				"try to reconnect to MQ Server...");
				consumer.reconnect();
				cont = false;
				logger.info("Reconnect successfully");
				consumer.stop();
				consumer.start();
			} catch (ResourceException re) {
				logger.error("ExceptionListenerImpl#onException : "
						+ "Reconnect attempt failed", re);
				cont = true;
			}
		}
	}
	
	
}