/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectionEventListenerHandler.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import java.util.ArrayList;

import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * This is the event listener of the connector.
 * 
 * @author Yacine TOUIL
 */
public class ConnectionEventListenerHandler {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * This is the list of the connection event listeners.
	 */
	private ArrayList list = new ArrayList();

	/**
	 * This is the ManagedConnection of this object.
	 */
	private ManagedConnectionImpl mc;

	/**
	 * Constructor of the class.
	 * 
	 * @param mc
	 *            the ManagedConnection for this Handler
	 */
	public ConnectionEventListenerHandler(ManagedConnectionImpl mc) {
		logger.debug("ConnectionEventListenerHandler#"
				+ "ConnectionEventListenerHandler(" + mc + ")");
		this.mc = mc;
	}

	/**
	 * Add a connection event into this object.
	 * 
	 * @param cel
	 *            the connection event listener to add.
	 */
	public void addConnectionEventListener(ConnectionEventListener cel) {
		logger.debug("ConnectionEventListenerHandler#"
				+ "addConnectionEventListener(" + cel + ")");
		list.add(cel);
	}

	/**
	 * Remove a connection event from this object.
	 * 
	 * @param cel
	 *            the connection event listener to remove.
	 */
	public void removeConnectionEventListener(ConnectionEventListener cel) {
		logger.debug("ConnectionEventListenerHandler#"
				+ "removeConnectionEventListener(" + cel + ")");
		list.remove(cel);
	}

	/**
	 * Send an event to This is the ManagedConnection of this object.
	 * 
	 * @param eventType
	 *            the type of the event.
	 * @param ex
	 *            the exception if there is.
	 * @param connectionHandle
	 *            the connection Handle if there is.
	 */
	public void sendEvent(int eventType, Exception ex, Object connectionHandle) {
		logger.debug("ConnectionEventListenerHandler#" + "sendEvent("
				+ eventType + "," + ex + "," + connectionHandle + ")");
		ArrayList list2 = (ArrayList) list.clone();
		ConnectionEvent connEvent = (ex != null) ? new ConnectionEvent(mc,
				eventType, ex) : new ConnectionEvent(mc, eventType);
		if (connectionHandle != null) {
			connEvent.setConnectionHandle(connectionHandle);
		}
		for (int i = 0; i < list2.size(); i++) {
			ConnectionEventListener cel = (ConnectionEventListener) list2
					.get(i);
			switch (eventType) {
			case ConnectionEvent.LOCAL_TRANSACTION_STARTED:
				cel.localTransactionStarted(connEvent);
				break;
			case ConnectionEvent.LOCAL_TRANSACTION_COMMITTED:
				cel.localTransactionCommitted(connEvent);
				break;
			case ConnectionEvent.LOCAL_TRANSACTION_ROLLEDBACK:
				cel.localTransactionRolledback(connEvent);
				break;
			case ConnectionEvent.CONNECTION_ERROR_OCCURRED:
				cel.connectionErrorOccurred(connEvent);
				break;
			case ConnectionEvent.CONNECTION_CLOSED:
				cel.connectionClosed(connEvent);
				break;
			default:
				throw new IllegalArgumentException(
						"Unknown Connection Event Type :" + eventType);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "list:" + list;
	}

}