/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectionManagerImpl.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnectionFactory;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * <code>ConnectionManager</code> implementation of the connector.
 * 
 * Creates a new <code>ManagedConnection</code> for every
 * <code>allocateConnection</code>.
 * 
 * @author Yacine TOUIL
 */
public class ConnectionManagerImpl implements ConnectionManager {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * alicate a new connection.
	 * 
	 * @param mcf
	 *            The Managed Conenction Factory
	 * @param connRequestInfo
	 *            The connexion request Informations
	 * @return the new allocated connection
	 * @throws ResourceException
	 *             generic exception if operation failed
	 */
	public Object allocateConnection(ManagedConnectionFactory mcf,
			ConnectionRequestInfo connRequestInfo) throws ResourceException {
		logger.debug("ConnectionManagerImpl#allocateConnection(" + mcf + ","
				+ connRequestInfo + ")");
		javax.resource.spi.ManagedConnection mc = mcf.createManagedConnection(
				null, connRequestInfo);
		return mc.getConnection(null, connRequestInfo);
	}

}