/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConnectionRequestInfoImpl.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnectionFactory;

import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;
import com.bull.jonasmqconnector.util.StringUtils;

/**
 * Specific request properties for the connector. userName, password and
 * clientId;
 * 
 * @author Yacine TOUIL
 */
public class ConnectionRequestInfoImpl implements ConnectionRequestInfo {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * The user name used for the connection.
	 */
	private String userName;

	/**
	 * The password used for the connection.
	 */
	private String password;

	/**
	 * The client id for this connection.
	 */
	private String clientID;

	/**
	 * The managed connection factory associated with this object.
	 */
	private ManagedConnectionFactory mcf;

	/**
	 * This is the constructor of the class.
	 */
	public ConnectionRequestInfoImpl() {
		logger.debug("ConnectionRequestInfoImpl#ConnectionRequestInfoImpl()");

	}

	/**
	 * This is the constructor of the class with initialisation params.
	 * 
	 * @param mcf
	 *            The managed connection factory associated with this object.
	 * @param userName
	 *            The user name used for the connection.
	 * @param password
	 *            The password used for the connection.
	 */
	public ConnectionRequestInfoImpl(ManagedConnectionFactory mcf,
			String userName, String password) {
		logger.debug("ConnectionRequestInfoImpl#ConnectionRequestInfoImpl("
				+ mcf + "," + userName + "" + password + ")");
		this.userName = userName;
		this.password = password;
		this.mcf = mcf;
		this.clientID = ((AbstractManagedConnectionFactory) mcf).getClientId();
	}

	/**
	 * Gets the Client ID
	 * 
	 * @return the Client ID
	 */
	public String getClientID() {
		logger.debug("ConnectionRequestInfoImpl#getClientID()");
		return clientID;
	}

	/**
	 * Sets the Client ID
	 * 
	 * @param clientID
	 *            the Client ID
	 */
	public void setClientID(String clientID) {
		logger.debug("ConnectionRequestInfoImpl#setClientID(" + clientID + ")");
		this.clientID = clientID;
	}

	/**
	 * Gets The password used for the connection.
	 * 
	 * @return The password used for the connection.
	 */
	public String getPassword() {
		logger.debug("ConnectionRequestInfoImpl#getPassword()");
		return password;
	}

	/**
	 * Sets the password used for the connection.
	 * 
	 * @param password
	 *            The password used for the connection.
	 */
	public void setPassword(String password) {
		logger.debug("ConnectionRequestInfoImpl#setPassword(" + password + ")");
		this.password = password;
	}

	/**
	 * Gets the user name for the connection
	 * 
	 * @return The user name used for the connection.
	 */
	public String getUserName() {
		logger.debug("ConnectionRequestInfoImpl#getUserName()");
		return userName;
	}

	/**
	 * Sets the user name used for the connection.
	 * 
	 * @param userName
	 *            The user name used for the connection.
	 */
	public void setUserName(String userName) {
		logger.debug("ConnectionRequestInfoImpl#setUserName(" + userName + ")");
		this.userName = userName;
	}

	/**
	 * Returns true if two ConnectionRequestInfo are equal; false otherwise
	 * 
	 * @param str1
	 *            <code>String</code>
	 * @param str2
	 *            <code>String</code>
	 * @return true if the two strings are equal false otherwise
	 */
	public boolean equals(Object obj) {
		logger.debug("ConnectionRequestInfoImpl#equals(" + obj + ")");
		if (obj == null || !(obj instanceof ConnectionRequestInfoImpl))
			return false;
		ConnectionRequestInfoImpl other = (ConnectionRequestInfoImpl) obj;
		return StringUtils.isEqual(this.userName, other.userName)
				&& StringUtils.isEqual(this.password, other.password);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "ConnectionRequestInfoImpl: " + userName + " " + password + " "
				+ clientID;
	}
}