/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ManagedTopicConnectionFactoryImpl.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.TopicConnectionFactory;
import javax.jms.XAConnection;
import javax.jms.XAConnectionFactory;
import javax.resource.spi.security.PasswordCredential;

import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * The Topic Managed Connection Factory
 * 
 * @author Yacine TOUIL
 */
public class ManagedTopicConnectionFactoryImpl extends
		AbstractManagedConnectionFactory {

	/**
	 * The default constructor
	 */
	public ManagedTopicConnectionFactoryImpl() {
		logger.debug("ManagedTopicConnectionFactoryImpl#"
				+ "ManagedTopicConnectionFactoryImpl()");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.bull.jonasmqconnector.outbound.AbstractManagedConnectionFactory#
	 *      getActualConnectionFactoryClassName()
	 */
	protected String getActualConnectionFactoryClassName() {
		logger.debug("ManagedTopicConnectionFactoryImpl#"
				+ "getActualConnectionFactoryClassName()");
		return Constants.MQ_XA_TCF_CLASSNAME;
	}

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.bull.jonasmqconnector.outbound.AbstractManagedConnectionFactory#
	 *      createConnection(javax.resource.spi.security.PasswordCredential,
	 *      javax.jms.ConnectionFactory)
	 */
	protected Connection createConnection(PasswordCredential pc,
			ConnectionFactory cf) throws JMSException {
		logger.debug("ManagedTopicConnectionFactoryImpl#createConnection(" + pc
				+ "," + cf + ")");
		if (pc != null) {
			return ((TopicConnectionFactory) cf).createConnection(pc
					.getUserName(), new String(pc.getPassword()));
		} else {
			return ((TopicConnectionFactory) cf).createConnection();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.bull.jonasmqconnector.outbound.AbstractManagedConnectionFactory#
	 *      createXAConnection(javax.resource.spi.security.PasswordCredential,
	 *      javax.jms.ConnectionFactory)
	 */
	protected XAConnection createXAConnection(PasswordCredential pc,
			XAConnectionFactory cf) throws JMSException {
		logger.debug("ManagedTopicConnectionFactoryImpl#createXAConnection("
				+ pc + "," + cf + ")");
		if (pc != null) {
			// return ((XATopicConnectionFactory)cf).createXAConnection(
			// pc.getUserName(), new String(pc.getPassword()));
			return ((XAConnectionFactory) cf).createXAConnection(pc
					.getUserName(), new String(pc.getPassword()));
		} else {
			// return ((XATopicConnectionFactory)cf).createXAConnection();
			return ((XAConnectionFactory) cf).createXAConnection();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.bull.jonasmqconnector.outbound.AbstractManagedConnectionFactory#
	 *      toString()
	 */
	public String toString() {
		return "ManagedTopicConnectionFactoryImpl: " + super.toString();
	}

}