/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TopicProxy.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.outbound;

import javax.jms.JMSException;
import javax.jms.Topic;

import com.bull.jonasmqconnector.util.Constants;
import com.bull.jonasmqconnector.util.LogUtils;
import com.bull.jonasmqconnector.util.Logger;

/**
 * A TopicProxy object encapsulates a provider-specific topic name.
 * 
 * @author Yacine TOUIL
 */
public class TopicProxy extends DestinationAdapter implements Topic {

	/**
	 * This is the logger of the class.
	 */
	private static Logger logger = LogUtils.getLogger();

	/**
	 * Gets the name of this topic.
	 * 
	 * @return the topic name
	 * @throws JMSException
	 *             if the JMS provider implementation of Topic fails to return
	 *             the topic name due to some internal error.
	 */
	public String getTopicName() throws JMSException {
		logger.debug("TopicProxy#getTopicName()");
		return ((Topic) this.getPhysicalDestination()).getTopicName();
	}

	/**
	 * Gets the class name of this topic.
	 * 
	 * @return the topic class name
	 */
	protected String getDestinationClassName() {
		logger.debug("TopicProxy#getDestinationClassName()");
		return Constants.MQ_TOPIC_CLASSNAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.bull.jonasmqconnector.outbound.DestinationAdapter#toString()
	 */
	public String toString() {
		try {
			return "TopicProxy: " + getTopicName() + " " + super.toString();
		} catch (Exception e) {
			return "TopicProxy: " + super.toString();
		}
	}

}