/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Constants.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.util;

/**
 * This class contains constants that are used by the classes of the connector
 * 
 * @author Yacine TOUIL, Kien THAI
 */
public class Constants {

	/**
	 * The Logger name
	 */
	public static final String LOGGER_NAME = "com.bull.jonasmqconnector";

	/**
	 * The class name for the MQ Queue
	 */
	public static final String MQ_QUEUE_CLASSNAME = "com.ibm.mq.jms.MQQueue";

	/**
	 * The class name for the MQ Topic
	 */
	public static final String MQ_TOPIC_CLASSNAME = "com.ibm.mq.jms.MQTopic";

	/**
	 * Unified session type
	 */
	public static final int JMS_SESSION = 0;

	/**
	 * Topic session type
	 */
	public static final int TOPIC_SESSION = 1;

	/**
	 * Queue session type
	 */
	public static final int QUEUE_SESSION = 2;

	/**
	 * The class name for the MQ Topic
	 */
	public static final String INACC_SYSTEM_PROP_KEY = "jonasmqconnector.inAppClientContainer";

	public static final String MQ_CONTEXT_FACTORY = "com.ibm.mq.jms.context.WMQInitialContextFactory";

	/**
	 * The client mode for the queue manager connection
	 */
	public static final String CLIENT_MODE = "client";

	public static final String MQ_CLIENT_MODE = "1";

	/**
	 * The binding mode for the queue manager connection
	 */
	public static final String BINDING_MODE = "binding";

	public static final String MQ_BINDING_MODE = "0";

	/**
	 * The default host for the queue manager
	 */
	public static final String DEFAULT_HOST = "localhost";

	/**
	 * The default port on which the queue manager listen
	 */
	public static final String DEFAULT_PORT = "1414";

	/**
	 * The default channel on which the queue manager accept the connections
	 */
	public static final String DEFAULT_CHANNEL = "SYSTEM.DEF.SVRCONN";

	/**
	 * The default authentication mode for queue manager JNDI repertory
	 */
	public static final String NONE_AUTHENTICATION_MODE = "none";

	/**
	 * The simple authentication mode for queue manager JNDI repertory
	 */
	public static final String SIMPLE_AUTHENTICATION_MODE = "simple";

	/**
	 * The default JNDI name of the JMS ConnectionFactory used for outbound
	 * communication
	 */
	public static final String CONN_FACT_NAME = "wsmqJCF";

	/**
	 * The default JNDI name of the JMS QueueConnectionFactory used for outbound
	 * communication
	 */
	public static final String QUEUE_CONN_FACT_NAME = "wsmqJQCF";

	/**
	 * The default JNDI name of the JMS TopicConnectionFactory used for outbound
	 * communication
	 */
	public static final String TOPIC_CONN_FACT_NAME = "wsmqJTCF";

	/**
	 * JMS Topic class name
	 */
	public static final String TOPIC_CLASSNAME = "javax.jms.Topic";

	/**
	 * JMS Queue class name
	 */
	public static final String QUEUE_CLASSNAME = "javax.jms.Queue";

	/**
	 * JMS MessageListener class name
	 */
	public static final String MSG_LISTENER_CLASSNAME = "javax.jms.MessageListener";

	/**
	 * JMS Message class name
	 */
	public static final String MESSAGE_CLASSNAME = "javax.jms.Message";

	/**
	 * JMS OnMessage method name
	 */
	public static final String ONMESSAGE_METHOD = "onMessage";

	/**
	 * MQ Connection Factory class name
	 */
	public static final String MQ_CF_CLASSNAME = "com.ibm.mq.jms.MQConnectionFactory";

	/**
	 * MQ Queue Connection Factory class name
	 */
	public static final String MQ_QCF_CLASSNAME = "com.ibm.mq.jms.MQQueueConnectionFactory";

	/**
	 * MQ Topic Connection Factory class name
	 */
	public static final String MQ_TCF_CLASSNAME = "com.ibm.mq.jms.MQTopicConnectionFactory";

	/**
	 * MQ XA Connection Factory class name
	 */
	public static final String MQ_XA_CF_CLASSNAME = "com.ibm.mq.jms.MQXAConnectionFactory";

	/**
	 * MQ XA Queue Connection Factory class name
	 */
	public static final String MQ_XA_QCF_CLASSNAME = "com.ibm.mq.jms.MQXAQueueConnectionFactory";

	/**
	 * MQ XA Topic Connection Factory class name
	 */
	public static final String MQ_XA_TCF_CLASSNAME = "com.ibm.mq.jms.MQXATopicConnectionFactory";

	public static final String TOPIC_NAME_PROPERTY = "BaseTopicName";

	public static final String QUEUE_NAME_PROPERTY = "BaseQueueName";

	/**
	 * 
	 */
	public static final String DURABLE = "Durable";

	public static final String NON_DURABLE = "NonDurable";

	public static final String AUTO_ACKNOWLEDGE = "Auto-acknowledge";

	public static final String DUPS_OK_ACKNOWLEDGE = "Dups-ok-acknowledge";

	public final static int DEFAULT_MAX_MESSAGES = 1;
	
	public final static int DEFAULT_MIN_POOL_SIZE = 1;

	public static final String SEPARATOR = ",";

	public static final String SETTER = "=";

	/**
	 * Default Non-managed Connection Factory JNDI Names
	 */
	public static final String JCF_JNDI_NAME = "wsmqJCF";

	/**
	 * Default Non-managed Topic Connection Factory JNDI Names
	 */
	public static final String JTCF_JNDI_NAME = "wsmqJTCF";

	/**
	 * Default Non-managed Queue Connection Factory JNDI Names
	 */
	public static final String JQCF_JNDI_NAME = "wsmqJQCF";

	/**
	 * Default performance event queue
	 */
	public static final String DEFAULT_PERFM_EVENT_QUEUE = 
		"SYSTEM.ADMIN.PERFM.EVENT";
	
	public static final String DEFAULT_CONFIG_FILENAME = "jonasmq.xml";
	
	public static final String DEFAULT_MBEAN_NAME = "jonasmq";
}
