/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JmsUtils.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.util;

import javax.jms.Connection;
import javax.jms.ConnectionConsumer;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.ServerSessionPool;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.XAConnection;
import javax.jms.XAConnectionFactory;
import javax.jms.XAQueueConnection;
import javax.jms.XAQueueConnectionFactory;
import javax.jms.XASession;
import javax.jms.XATopicConnection;
import javax.jms.XATopicConnectionFactory;

/**
 * An util class used for create JMS connections, sessions and connection
 * consumers
 * 
 * @author Kien THAI
 * 
 */
public class JmsUtils {

	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	public static Connection createConnection(ConnectionFactory cf,
			boolean isTopic) throws JMSException {
		logger.debug("JmsUtils#createConnection(" + cf + "," + isTopic + ")");
		Connection con = null;

		if (isTopic)
			con = ((TopicConnectionFactory) cf).createTopicConnection();
		else
			con = ((QueueConnectionFactory) cf).createQueueConnection();
		return con;
	}

	public static Connection createConnection(ConnectionFactory cf,
			boolean isTopic, String user, String password) throws JMSException {
		logger.debug("JmsUtils#createConnection(" + cf + "," + isTopic + ","
				+ user + "," + password + ")");
		Connection con = null;
		if (password != null) {
			if (isTopic)
				con = ((TopicConnectionFactory) cf).createTopicConnection(user,
						password);
			else
				con = ((QueueConnectionFactory) cf).createQueueConnection(user,
						password);
		} else {
			if (isTopic)
				con = ((TopicConnectionFactory) cf).createTopicConnection();
			else
				con = ((QueueConnectionFactory) cf).createQueueConnection();
		}
		return con;
	}

	public static Session createSession
	(Connection con, boolean isTopic, int acknowledgeMode)
			throws JMSException {
		logger.debug("JmsUtils#createSession(" + con + "," + isTopic + ")");
		Session sess = null;
		if (isTopic)
			sess = ((TopicConnection) con).createTopicSession(false,
					acknowledgeMode);
		else
			sess = ((QueueConnection) con).createQueueSession(false,
					acknowledgeMode);
		return sess;
	}

	public static XAConnection createXAConnection(XAConnectionFactory xacf,
			boolean isTopic) throws JMSException {
		logger.debug("JmsUtils#createXAConnection(" + xacf + "," + isTopic
				+ ")");
		XAConnection xac = null;

		if (isTopic)
			xac = ((XATopicConnectionFactory) xacf).createXATopicConnection();
		else
			xac = ((XAQueueConnectionFactory) xacf).createXAQueueConnection();
		return xac;
	}

	public static Connection createXAConnection(XAConnectionFactory xacf,
			boolean isTopic, String user, String password) throws JMSException {
		logger.debug("JmsUtils#createXAConnection(" + xacf + "," + isTopic
				+ "," + user + "," + password + ")");
		XAConnection xac = null;
		if (password != null) {
			if (isTopic)
				xac = ((XATopicConnectionFactory) xacf)
						.createXATopicConnection(user, password);
			else
				xac = ((XAQueueConnectionFactory) xacf)
						.createXAQueueConnection(user, password);
		} else {
			if (isTopic)
				xac = ((XATopicConnectionFactory) xacf)
						.createXATopicConnection();
			else
				xac = ((XAQueueConnectionFactory) xacf)
						.createXAQueueConnection();
		}
		return xac;
	}

	public static XASession createXASession
	(XAConnection con, boolean isTopic)
			throws JMSException {
		logger.debug("JmsUtils#createXASession(" + con + "," + isTopic + ")");
		XASession result = null;
		if (isTopic)
			result = ((XATopicConnection) con).createXATopicSession();
		else
			result = ((XAQueueConnection) con).createXAQueueSession();
		return result;
	}

	public static ConnectionConsumer createConnectionConsumer(Connection con,
			boolean isTopic, Destination dest, String messageSelector,
			ServerSessionPool ssPool, int maxMessages) throws JMSException {
		logger.debug("JmsUtils#createConnectionConsumer(" + con + "," + isTopic
				+ "," + dest + "," + messageSelector + "," + ssPool + ","
				+ maxMessages + "," + ")");
		ConnectionConsumer consumer = null;
		if (isTopic)
			consumer = ((TopicConnection) con).createConnectionConsumer(
					(Topic) dest, messageSelector, ssPool, maxMessages);
		else
			consumer = ((QueueConnection) con).createConnectionConsumer(
					(Queue) dest, messageSelector, ssPool, maxMessages);
		return consumer;
	}
}