/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JndiObjectBuilder.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.util;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;

/**
 * An util class used for create object by looking up from JNDI repertory
 * 
 * @author Kien THAI
 * 
 */
public class JndiObjectBuilder {
	/**
	 * Logger for the class
	 */
	private static Logger logger = LogUtils.getLogger();

	private InitialContext context = null;

	private String jndiName = null;

	public JndiObjectBuilder() {

	}

	public JndiObjectBuilder(String jndiName) throws ResourceException {
		logger.debug("JndiObjectBuilder#JndiObjectBuilder(" + jndiName + ")");
		this.jndiName = jndiName;
		try {
			// Creating an InitialContext.
			context = new InitialContext();
		} catch (NamingException ne) {
			throw ExceptionUtils.newResourceException(
					"Can not create JNDI context", ne);
		}

	}

	public Object build() throws ResourceException {
		logger.debug("JndiObjectBuilder#build()");
		Object obj = null;
		try {

			obj = context.lookup(jndiName);

		} catch (NamingException ne) {
			throw ExceptionUtils.newResourceException("Can not lookup : "
					+ jndiName, ne);
		}
		return obj;
	}
}