/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Logger.java 10254 2007-04-23 11:31:11Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.util;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * logging interface to monolog
 * @author Yacine TOUIL
 *
 */
public class Logger {
	
	/**
	 * Monolog Logger
	 */
	private static org.objectweb.util.monolog.api.Logger logger = 
		Log.getLogger(Constants.LOGGER_NAME);

	/**
	 * Local instance
	 */
	private static Logger instance = new Logger();
	
	/**
	 * Get instance method
	 */
	public static Logger getInstance() {
		return instance;
	}
	
	/**
	 * log info
	 * @param obj
	 */
	public void info(Object obj) {
		logger.log(BasicLevel.INFO, obj);
	}
	
	/**
	 * log warn
	 * @param obj
	 */
	public void warn(Object obj) {
		logger.log(BasicLevel.LEVEL_WARN, obj);
	}
	
	/**
	 * log debug
	 * @param obj
	 */
	public void debug(Object obj) {
		logger.log(BasicLevel.DEBUG, obj);
	}
	
	/**
	 * log error
	 * @param obj
	 */
	public void error(Object obj) {
		logger.log(BasicLevel.ERROR, obj);
	}

	/**
	 * log info
	 * @param obj
	 */
	public void info(Object obj, Throwable t) {
		logger.log(BasicLevel.INFO, obj, t);
	}
	
	/**
	 * log warn
	 * @param obj
	 */
	public void warn(Object obj, Throwable t) {
		logger.log(BasicLevel.WARN, obj, t);
	}
	
	/**
	 * log debug
	 * @param obj
	 */
	public void debug(Object obj, Throwable t) {
		logger.log(BasicLevel.DEBUG, obj, t);
	}
	
	/**
	 * log error
	 * @param obj
	 */
	public void error(Object obj, Throwable t) {
		logger.log(BasicLevel.ERROR, obj, t);
	}
	
	/**
	 * Check debug level
	 * @param obj
	 */
	public boolean isDebug() {
		return logger.isLoggable(BasicLevel.DEBUG);
	}

	/**
	 * Check info level
	 * @param obj
	 */
	public boolean isInfo() {
		return logger.isLoggable(BasicLevel.INFO);
	}

	/**
	 * Check warn level
	 * @param obj
	 */
	public boolean isWarn() {
		return logger.isLoggable(BasicLevel.WARN);
	}

}
