/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MBeanDestinationTest.java 10249 2007-04-23 11:22:14Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.test.mbean;

import java.io.FileInputStream;
import java.util.Properties;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import junit.framework.TestCase;

/**
 * This Unit Test 2 : Connector MBean Test
 *  
 * @author Yacine TOUIL
 */
public class MBeanDestinationTest extends TestCase {
	
	/**
	 * The configuration filename
	 */
	private final static String CONFIG_FILENAME = "config.properties";
	
	/**
	 * Properties in the config file
	 */
	private Properties properties;

	/**
	 * initialized or not
	 */
	private static boolean initialized = false;
	
	/**
	 * Connector JMX
	 */
	private JMXConnector connector;

	/**
	 * MBean Connection
	 */
	private static MBeanServerConnection mBeanConn;
	
	/**
	 * Connector JMX ObjectName
	 */
	private static ObjectName connectorON;
	
	/**
	 * Queue JMX ObjectName
	 */
	private static ObjectName queueON;

	/**
	 * This method initialize the test
	 */
	private void initialize() {
		properties = new Properties();
		try {
			properties.load(new FileInputStream(CONFIG_FILENAME));
		} catch(Exception e) {
			e.printStackTrace();
			// put default values
			properties.setProperty("JMX_URL", 
			"service:jmx:rmi:///jndi/rmi://localhost:1099/jrmpconnector_jonas");
		}
		
		// Get InitialContext
		try {
			String jmxURL = properties.getProperty("JMX_URL");
			JMXServiceURL connURL = new JMXServiceURL(jmxURL);
			connector = JMXConnectorFactory.newJMXConnector(connURL, null);
			connector.connect();
			mBeanConn = connector.getMBeanServerConnection();
			connectorON = new ObjectName("jonas:type=JonasMQConnector," +
			"name=jonasmq");
			String[] signature = {"boolean", "java.lang.String",
			"java.lang.String"};
			Object[] params = {Boolean.FALSE, "TestQueue", ""};
			mBeanConn.invoke(connectorON, "createDestination", params, signature);
			queueON = new ObjectName("jonas:type=MQDestination,name=TestQueue" +
					",JonasMQConnector=jonasmq");

            initialized = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method initialize the test
	 */
	protected void setUp() {
		if (!initialized)
			initialize();
	}

	/**
	 * This method finalize the test
	 */
	protected void tearDown() throws Exception {
		//connector.close();
	}

	/**
	 * This Unit Test Clear Queue
	 */
	public void testClearQueue() {
		System.out.println("MBean Test : ClearQueue");
		boolean done = false;
		try {
			String[] signature = {};
			Object[] params = {};
			mBeanConn.invoke(queueON, "clearQueue", params, signature);
			done = true;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		assertEquals(done, true);
	}

	/**
	 * This Unit Test Get Current Depth
	 */
	public void testGetCurrentDepth() {
		System.out.println("MBean Test : GetCurrentDepth");
		boolean done = false;
		try {
			mBeanConn.getAttribute(queueON, "CurrentDepth");
			done = true;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		assertEquals(done, true);
	}

	/**
	 * This Unit Test get InputCount
	 */
	public void testGetInputCount() {
		System.out.println("MBean Test : getInputCount");
		boolean done = false;
		try {
			mBeanConn.getAttribute(queueON, "InputCount");
			done = true;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		assertEquals(done, true);
	}

	/**
	 * This Unit Test get OutputCount
	 */
	public void testGetOutputCount() {
		System.out.println("MBean Test : getOutputCount");
		boolean done = false;
		try {
			mBeanConn.getAttribute(queueON, "OutputCount");
			done = true;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		assertEquals(done, true);
	}
}