/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MsgSourceClient.java 10249 2007-04-23 11:22:14Z waeselynck $
 * --------------------------------------------------------------------------
 */

// MsgSourceClient.java
// Client for accessing session bean MsgSource 
// and send messages from the session bean to an MDB
package com.bull.jonasmqconnector.test.msgsource;

import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import com.bull.jonasmqconnector.test.util.Constants;
import com.bull.jonasmqconnector.test.util.Util;

/**
 * A JOnAS Client Application for mesuring time to transfer messages from a
 * Session Bean to an MDB through a Topic. Messages are sent one after other.
 * This program test also the capacity to create a Destination through JMX
 * Server
 * 
 * The MDB listen to the topic in non-durable mode and receive the messages in
 * transactionnal mode
 * 
 * @author Kien THAI
 * 
 */
public class MsgSourceClient {

	private static Context initialContext = null;

	private static MsgSourceHome BeanHome = null;

	// private static UserTransaction utx = null;

	private static QueueConnectionFactory qcf = null;

	static Queue replyQueue = null;

	// Number of messages for test
	private static int nbmess = 100;

	static long TIME_OUT = 60000;
	
	public static void main(String[] args) {
		try {
			if (args.length == 1)
			{
				nbmess = Integer.parseInt(args [0]);
			} else if (args.length == 2) {
				
				nbmess = Integer.parseInt(args [0]);
				TIME_OUT = Long.parseLong(args[1]);
			}
		} catch (Exception e) {
			
		}
		System.out.println("Send " + 
				nbmess + " messages to MDB and wait in " + TIME_OUT + 
		" milliseconds for the reply.");
		
		System.out.println("===============================================");
		System.out.println("This test deploy an MDB which listen to a TOPIC \n" +
						   "in REMOTE MQ SERVER, \n" +
						   "with a TRANSACTIONAL Connection.\n" +
						   "A stand-alone Java client progam will activate \n" +
						   "a SESSION BEAN to send 100 messages sequently \n" +
						   "by a transactional connection to the topic \n" + 
						   "and listen to a queue to receive the reply");
		System.out.println("===============================================");
		// Get InitialContext
		try {
			initialContext = new InitialContext();
		} catch (NamingException e) {
			e.printStackTrace();
			System.exit(2);
		}

		test();

		result();
	}

	public static void test() {

		try {
			System.out
					.println("Initialize ConnectionFactoties and Destinations");
			// create or lookup the Reply Queue through its JNDI name
			replyQueue = Util.createQueueIfNotExists(
					Constants.DEFAULT_CONNECTOR_MBEAN_NAME,
					Constants.JMX_URL, initialContext,
					Constants.SB_MDB_REPLY_QUEUE_NAME, "");

			
		} catch (Exception e) {
			
			System.out.println("Fail to create destination -> use JOnAS console" +
					"to create following destination and retry the test: ");
			System.exit(2);
		}
		
		System.out.println("Result Queue: " + Constants.SB_MDB_REPLY_QUEUE_NAME);

		// For starting transactions from client: get UserTransaction
		// System.out.println("Getting a UserTransaction object from JNDI");
		// try {
		// utx = (UserTransaction)
		// initialContext.lookup("javax.transaction.UserTransaction");
		// } catch (Exception e) {
		// System.err.println("Cannot lookup UserTransaction: " + e);
		// System.exit(2);
		// }

		// Lookup MsgSource bean home
		try {
			BeanHome = (MsgSourceHome) PortableRemoteObject
					.narrow(initialContext.lookup("MsgSourceHome"),
							MsgSourceHome.class);
		} catch (Exception e) {
			System.err.println("Cannot lookup MsgSourceHome: " + e);
			System.exit(2);
		}

		// MsgSource creation
		MsgSource aJmsBean = null;
		try {
			System.out.println("Creating an MsgSource bean");
			aJmsBean = BeanHome.create();
		} catch (Exception e) {
			System.err.println("Cannot create MsgSource: " + e);
			System.exit(2);
		}

		// Calling MsgSource method that sends multiple messages
		try {
			// utx.begin();
			System.out.println("Start to send " + nbmess + " messages");
			aJmsBean.sendMsg(nbmess);
			// utx.commit();
		} catch (Exception e) {
			System.err.println("Exception .... : " + e);
			e.printStackTrace();
			System.exit(2);
		}

	}

	public static void result() {
		try {
			// lookup the Client QueueConnectionFactory through its JNDI name
			qcf = (QueueConnectionFactory) initialContext
					.lookup(Constants.CLIENT_QCF);
			System.out
					.println("Message Source Client: qcf = " + qcf.toString());

			QueueReceiver qReceiver;
			QueueConnection qc = qcf.createQueueConnection();
			System.out.println("Message Source Client: qc = " + qc.toString());
			QueueSession qSession = qc.createQueueSession(false,
					Session.AUTO_ACKNOWLEDGE);
			qc.start();

			qReceiver = qSession.createReceiver(replyQueue);

			System.out
					.println("Waiting for reply message from the test MDB with the timeout of "
							+ TIME_OUT + " miliseconds...");
			
			TextMessage msg = (TextMessage) qReceiver.receive(TIME_OUT);

			int numberMsgReceived = msg.getIntProperty(Constants.SB_MDB_NUMBER_MSG_RECEIVED);
			long sendTime = msg.getLongProperty(Constants.SEND_TIME_KEY);

//			if () {
//				System.out.println("Test success!");
//			} else {
//				System.out.println("Test fail! "
//						+ "Clear result queue "
//						+ Constants.SB_MDB_REPLY_QUEUE_NAME
//						+ " and try the test again");
//			}
			System.out.println("Test success!");
			System.out.println("Number of messages received is : " + numberMsgReceived);
			
			System.out
					.println("Time to send " + numberMsgReceived
							+ " messages from a Session Bean to an MDB is: "
							+ sendTime);

			qSession.close();
			qc.close();
			System.in.read();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			System.out.println("Test fail! Can not receive the reply message");
			e.printStackTrace();
		}
	}
}