/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: OutboundlTest.java 10249 2007-04-23 11:22:14Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.test.outbound;

import java.io.FileInputStream;
import java.util.Hashtable;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;

import junit.framework.TestCase;

import com.bull.jonasmqconnector.test.util.Constants;
import com.bull.jonasmqconnector.test.util.Util;

/**
 * This Unit Test create a connexions and use it
 *  
 * @author Yacine TOUIL
 */
public class OutboundlTest extends TestCase {
	
	/**
	 * the number of message to send in many message tests
	 */
	private final static int NBR_MSG_SEND = 100;
	/**
	 * The configuration filename
	 */
	private final static String CONFIG_FILENAME = "config.properties";
	
	/**
	 * Properties in the config file
	 */
	private Properties properties;

	/**
	 * The context of JOnAS
	 */
	private static Context ictx = null;

	/**
	 * JNDI name of the JMS connection factory 
	 */
	private static String jmsConFactName = "JCF";
	
	/**
	 * JNDI name of the Topic connection factory
	 */
	private static String topicConFactName = "JTCF";

	/**
	 * JNDI name of the Queue connection factory
	 */
	private static String queueConFactName = "JQCF";

	/**
	 * The JMS connection factory 
	 */
	private static ConnectionFactory cf = null;

	/**
	 * The Topic connection factory
	 */
	private static TopicConnectionFactory tcf = null;

	/**
	 * The Queue connection factory
	 */
	private static QueueConnectionFactory qcf = null;

	/**
	 * JNDI name of the Topic
	 */
	private static String topicName = "SAMPLE_TOPIC";

	/**
	 * JNDI name of the Queue
	 */
	private static String queueName = "SAMPLE_QUEUE";

	/**
	 * a Topic
	 */
	private static Topic topic = null;

	/**
	 * a Queue
	 */
	private static Queue queue = null;

	/**
	 * JNDI name of the Topic
	 */
	private static String persistentTopicName = "PERSISTENTE_SAMPLE_TOPIC";

	/**
	 * JNDI name of the Queue
	 */
	private static String persistentQueueName = "PERSISTENTE_SAMPLE_QUEUE";

	/**
	 * a Topic
	 */
	private static Topic persistentTopic = null;

	/**
	 * a Queue
	 */
	private static Queue persistentQueue = null;
	/**
	 * initialized or not
	 */
	private static boolean initialized = false;
	
	/**
	 * This method initialize the test
	 */
	private void initialize() {
		properties = new Properties();
		try {
			properties.load(new FileInputStream(CONFIG_FILENAME));
		} catch(Exception e) {
			e.printStackTrace();
			// put default values
			properties.setProperty("INITIAL_CONTEXT_FACTORY",
				"org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");
			properties.setProperty("PROVIDER_URL", "rmi://localhost:1099");
			properties.setProperty("JMX_URL", 
			"service:jmx:rmi:///jndi/rmi://localhost:1099/jrmpconnector_jonas");
		}
		
		// Get InitialContext
		 Hashtable env = new Hashtable();
		 env.put(Context.INITIAL_CONTEXT_FACTORY, 
				 properties.get("INITIAL_CONTEXT_FACTORY"));
		 env.put(Context.PROVIDER_URL, properties.get("PROVIDER_URL"));
		try {
			ictx = new InitialContext(env);
			// lookup the ConnectionFactory through its JNDI name
			cf = (ConnectionFactory) ictx.lookup(jmsConFactName);
			System.out.println("JMS client: cf = " + cf.toString());

			tcf = (TopicConnectionFactory) ictx.lookup(topicConFactName);
			System.out.println("JMS client: tcf = " + tcf.toString());
			
			qcf = (QueueConnectionFactory) ictx.lookup(queueConFactName);
			System.out.println("JMS client: qcf = " + qcf.toString());

			String jmxURL = properties.getProperty("JMX_URL");
			topic = Util.createTopicIfNotExists(
					Constants.DEFAULT_CONNECTOR_MBEAN_NAME,
					jmxURL, ictx, topicName, "");
			queue = Util.createQueueIfNotExists(
					Constants.DEFAULT_CONNECTOR_MBEAN_NAME,
					jmxURL, ictx, queueName, "");
			
			persistentTopic = Util.createTopicIfNotExists(
					Constants.DEFAULT_CONNECTOR_MBEAN_NAME,
					jmxURL, ictx, 
					persistentTopicName, "");
			persistentQueue = Util.createQueueIfNotExists(
					Constants.DEFAULT_CONNECTOR_MBEAN_NAME,
					jmxURL, ictx, 
					persistentQueueName, "");

            initialized = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method initialize the test
	 */
	protected void setUp() {
		if (!initialized)
			initialize();
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Topic)
	 */
	public void testJMSTopic() {
		System.out.println("JMS Test (Topic)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) topic);
          MessageProducer mp = session.createProducer((Destination) topic);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          mp.send(msg);
          TextMessage m = (TextMessage)mc.receive();
          msgReceive = m.getText();
          System.out.println("Message = " + msgReceive);
          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Topic)
	 */
	public void testJMSPersistenceTopic() {
		System.out.println("JMS Persistence Test (Topic)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) 
        		  persistentTopic);
          MessageProducer mp = session.createProducer((Destination) 
        		  persistentTopic);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          mp.send(msg);
          TextMessage m = (TextMessage)mc.receive();
          msgReceive = m.getText();
          System.out.println("Message = " + msgReceive);
          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Queue)
	 */
	public void testJMSQueue() {
		System.out.println("JMS Test (Queue)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) queue);
          MessageProducer mp = session.createProducer((Destination) queue);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          mp.send(msg);
          TextMessage m = (TextMessage)mc.receive();
          msgReceive = m.getText();
          System.out.println("Message = " + msgReceive);
          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Queue)
	 */
	public void testJMSPersistenceQueue() {
		System.out.println("JMS Persistence Test (Queue)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) 
        		  persistentTopic);
          MessageProducer mp = session.createProducer((Destination) 
        		  persistentTopic);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          mp.send(msg);
          TextMessage m = (TextMessage)mc.receive();
          msgReceive = m.getText();
          System.out.println("Message = " + msgReceive);
          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a Topic connexions and use it (Topic)
	 */
	public void testTopic() {
		System.out.println("Topic Test");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
            TopicConnection conn = tcf.createTopicConnection();
            TopicSession session = conn.createTopicSession(false, 
            		Session.AUTO_ACKNOWLEDGE);
            conn.start();
            MessageConsumer mc = session.createConsumer((Destination) topic);
            MessageProducer mp = session.createProducer((Destination) topic);
            TextMessage msg = session.createTextMessage();
            msg.setText(msgSend);
            mp.send(msg);
            TextMessage m = (TextMessage)mc.receive();
            msgReceive = m.getText();
            System.out.println("Message = " + msgReceive);
            session.close();
            conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a Topic connexions and use it (Topic)
	 */
	public void testTopicPersistence() {
		System.out.println("Persistent Topic Test");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
            TopicConnection conn = tcf.createTopicConnection();
            TopicSession session = conn.createTopicSession(false, 
            		Session.AUTO_ACKNOWLEDGE);
            conn.start();
            MessageConsumer mc = session.createConsumer((Destination) 
            		persistentTopic);
            MessageProducer mp = session.createProducer((Destination) 
            		persistentTopic);
            TextMessage msg = session.createTextMessage();
            msg.setText(msgSend);
            mp.send(msg);
            TextMessage m = (TextMessage)mc.receive();
            msgReceive = m.getText();
            System.out.println("Message = " + msgReceive);
            session.close();
            conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a Queue connexions and use it (Queue)
	 */
	public void testQueue() {
		System.out.println("Queue Test");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
            QueueConnection conn = qcf.createQueueConnection();
            QueueSession session = conn.createQueueSession(false, 
            		Session.AUTO_ACKNOWLEDGE);
            MessageConsumer mc = session.createConsumer((Destination) queue);
            MessageProducer mp = session.createProducer((Destination) queue);
            conn.start();
            TextMessage msg = session.createTextMessage();
            msg.setText(msgSend);
            mp.send(msg);
            TextMessage m = (TextMessage)mc.receive();
            msgReceive = m.getText();
            System.out.println("Message = " + msgReceive);
            session.close();
            conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a Queue connexions and use it (Queue)
	 */
	public void testQueuePersistence() {
		System.out.println("Persistent Queue Test");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
            QueueConnection conn = qcf.createQueueConnection();
            QueueSession session = conn.createQueueSession(false, 
            		Session.AUTO_ACKNOWLEDGE);
            MessageConsumer mc = session.createConsumer((Destination) 
            		persistentQueue);
            MessageProducer mp = session.createProducer((Destination) 
            		persistentQueue);
            conn.start();
            TextMessage msg = session.createTextMessage();
            msg.setText(msgSend);
            mp.send(msg);
            TextMessage m = (TextMessage)mc.receive();
            msgReceive = m.getText();
            System.out.println("Message = " + msgReceive);
            session.close();
            conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Topic)
	 */
	public void testJMSTopicManyMessage() {
		System.out.println("JMS Test (Topic)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) topic);
          MessageProducer mp = session.createProducer((Destination) topic);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          TextMessage m;
          boolean equals = true;
          long startTime = System.currentTimeMillis();
          int i;
          for (i = 0; i < NBR_MSG_SEND; i++) {
              mp.send(msg);
              m = (TextMessage)mc.receive();
              msgReceive = m.getText();
              if (!msgSend.equals(msgReceive))
            	  break;
          }
          long stopTime = System.currentTimeMillis();
          System.out.println("Message = " + msgReceive+", "+i+" messsage " +
          		"received in "+(stopTime - startTime)+" ms");
          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Queue)
	 */
	public void testJMSQueueManyMessage() {
		System.out.println("JMS Test (Queue)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) queue);
          MessageProducer mp = session.createProducer((Destination) queue);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          TextMessage m;
          boolean equals = true;
          long startTime = System.currentTimeMillis();
          int i;
          for (i = 0; i < NBR_MSG_SEND; i++) {
              mp.send(msg);
              m = (TextMessage)mc.receive();
              msgReceive = m.getText();
              if (!msgSend.equals(msgReceive))
            	  break;
          }
          long stopTime = System.currentTimeMillis();
          System.out.println("Message = " + msgReceive+", "+i+" messsage " +
          		"received in "+(stopTime - startTime)+" ms");

          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Topic)
	 */
	public void testJMSTopicPersistentManyMessage() {
		System.out.println("JMS Test (Topic)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) 
        		  persistentTopic);
          MessageProducer mp = session.createProducer((Destination) 
        		  persistentTopic);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          TextMessage m;
          boolean equals = true;
          long startTime = System.currentTimeMillis();
          int i;
          for (i = 0; i < NBR_MSG_SEND; i++) {
              mp.send(msg);
              m = (TextMessage)mc.receive();
              msgReceive = m.getText();
              if (!msgSend.equals(msgReceive))
            	  break;
          }
          long stopTime = System.currentTimeMillis();
          System.out.println("Message = " + msgReceive+", "+i+" messsage " +
          		"received in "+(stopTime - startTime)+" ms");
          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

	/**
	 * This Unit Test create a JMS connexions and use it (Queue)
	 */
	public void testJMSQueuePersistentManyMessage() {
		System.out.println("JMS Test (Queue)");
        String msgSend = "Hello World!";
        String msgReceive = null;
		try {
          Connection conn = cf.createConnection();
          Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
          conn.start();
          MessageConsumer mc = session.createConsumer((Destination) 
        		  persistentQueue);
          MessageProducer mp = session.createProducer((Destination) 
        		  persistentQueue);
          TextMessage msg = session.createTextMessage();
          msg.setText(msgSend);
          TextMessage m;
          boolean equals = true;
          long startTime = System.currentTimeMillis();
          int i;
          for (i = 0; i < NBR_MSG_SEND; i++) {
              mp.send(msg);
              m = (TextMessage)mc.receive();
              msgReceive = m.getText();
              if (!msgSend.equals(msgReceive))
            	  break;
          }
          long stopTime = System.currentTimeMillis();
          System.out.println("Message = " + msgReceive+", "+i+" messsage " +
          		"received in "+(stopTime - startTime)+" ms");

          session.close();
          conn.close();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			e.printStackTrace();
		}
		assertEquals(msgSend, msgReceive);
	}

}