/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SbMdbTestBean.java 10249 2007-04-23 11:22:14Z waeselynck $
 * --------------------------------------------------------------------------
 */

//SbMdbTestBean.java
//Message Driven bean
package com.bull.jonasmqconnector.test.sbmdb;

import java.util.HashSet;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.bull.jonasmqconnector.test.util.Constants;

/**
 * A Message Driven Bean used for testing message reception and mesure the time
 * to transfer message from a client through a Topic
 * 
 * @author Kien THAI
 * 
 */
public class SbMdbTestBean implements MessageDrivenBean, MessageListener {

	private transient MessageDrivenContext mdbContext;

	// JNDI will be used to get the JMS administered objects
	static Context ictx = null;

	static QueueSender sender = null;

	private QueueConnectionFactory qcf = null;

	private Queue queue = null;

	private static HashSet idSet = new HashSet();

	// ------------------------------------------------------------------
	// MessageDrivenBean implementation
	// ------------------------------------------------------------------

	/**
	 * Default constructor
	 */
	public SbMdbTestBean() {
	}

	/**
	 * Set the associated context. The container call this method after the
	 * instance creation. The enterprise Bean instance should store the
	 * reference to the context object in an instance variable. This method is
	 * called with no transaction context.
	 * 
	 * @param MessageDrivenContext
	 *            A MessageDrivenContext interface for the instance.
	 * @throws EJBException
	 *             Thrown by the method to indicate a failure caused by a
	 *             system-level error.
	 */

	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		System.out.println("SbMdbTestBean setMessageDrivenContext");
		mdbContext = ctx;
	}

	/**
	 * A container invokes this method before it ends the life of the
	 * message-driven object. This happens when a container decides to terminate
	 * the message-driven object. This method is called with no transaction
	 * context.
	 * 
	 * @throws EJBException
	 *             Thrown by the method to indicate a failure caused by a
	 *             system-level error.
	 */
	public void ejbRemove() {
		System.out.println("SbMdbTestBean ejbRemove");
		idSet.clear();
	}

	/**
	 * The Message driven bean must define an ejbCreate methods with no args.
	 */
	public void ejbCreate() {
		System.out.println("SbMdbTestBean ejbCreate");
	}

	/**
	 * onMessage method
	 */
	public void onMessage(Message message) {
		System.out.println("SbMdbTestBean onMessage");
		try {
			int msgId = message.getIntProperty(Constants.SB_MDB_MSG_ID);
			System.out.println("Received message " + msgId);
			idSet.add(new Integer(msgId));
			
			boolean end = message.getBooleanProperty(Constants.END_KEY);
			if (end) {
				
				replyMsg(message, idSet.size());
				
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void replyMsg(Message message, int numberMsgReceived) {
		long endSendTime = System.currentTimeMillis();
		System.out.println("Send reply message");
		// Get InitialContext
		try {
			ictx = new InitialContext();
			// lookup the TopicConnectionFactory through its JNDI name
			qcf = (QueueConnectionFactory) ictx.lookup(Constants.M_QCF);
			System.out.println("Test MDB: qcf = " + qcf.toString());
			// lookup the Topic through its JNDI name
			queue = (Queue) ictx.lookup(Constants.SB_MDB_REPLY_QUEUE_NAME);
		} catch (NamingException e) {
			e.printStackTrace();
		}

		QueueConnection qc = null;
		QueueSession session = null;
		try {
			qc = qcf.createQueueConnection();
			System.out.println("Test MDB: qc = " + qc.toString());
			session = qc.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			sender = session.createSender(queue);
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			TextMessage replyMessage = session.createTextMessage();

			long beginSendTime = message
					.getLongProperty(Constants.BEGIN_SEND_TIME_KEY);
			long sendTime = endSendTime - beginSendTime;

			System.out.println("Last message received: " + numberMsgReceived);

			replyMessage.setLongProperty(Constants.SEND_TIME_KEY, sendTime);
			replyMessage.setIntProperty(Constants.SB_MDB_NUMBER_MSG_RECEIVED, numberMsgReceived);

			sender.send(replyMessage);

			session.close();

			qc.close();

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			idSet.clear();
		}
	}
}