/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TMdbTest.java 10249 2007-04-23 11:22:14Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.test.tmdb;

import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.bull.jonasmqconnector.test.util.Constants;
import com.bull.jonasmqconnector.test.util.Util;

/**
 * A JOnAS Client Application for mesuring time to transfer message from a
 * client to an MDB through a Topic. Messages are sent one after other.
 * 
 * The MDB listen to the topic in non-durable mode and receive the messages in
 * transactionnal mode
 * 
 * This program test also the capacity to create a Destination through JMX
 * Server
 * 
 * @author Kien THAI
 * 
 */
public class TMdbTest {

	static Context ictx = null;

	static TopicConnectionFactory tcf = null;

	static QueueConnectionFactory qcf = null;

	static TopicPublisher tp = null;

	static Topic topic = null;

	static Queue replyQueue = null;

	// JNDI name of the Topic
	static String topicName = "T_MDB_TEST_TOPIC";

	// Number of messages for test
	public static int nbmess = 100;

	// Time out for receiving reply message
	static long TIME_OUT = 60000;
	
	public static void main(String[] args) {
		try {
			if (args.length == 1)
			{
				nbmess = Integer.parseInt(args [0]);
			} else if (args.length == 2) {
				
				nbmess = Integer.parseInt(args [0]);
				TIME_OUT = Long.parseLong(args[1]);
			}
		} catch (Exception e) {
			
		}
		System.out.println("Send " + 
				nbmess + " messages to MDB and wait in " + TIME_OUT + 
		" milliseconds for the reply.");

		System.out.println("===============================================");
		System.out.println("This test deploy an MDB which listen to a TOPIC \n" +
						   "in LOCAL MQ SERVER with a TRANSACTIONAL Connection.\n" +
						   "A stand-alone Java client progam will send \n" +
						   "100 messages sequently to the topic \n" +
						   "and listen to a queue to receive the reply");
		System.out.println("===============================================");
		String initTestKey = null;
		try {
			System.out
					.println("Initialize ConnectionFactoties and Destinations");

			// Get InitialContext
			ictx = new InitialContext();

			// lookup the TopicConnectionFactory through its JNDI name
			tcf = (TopicConnectionFactory) ictx.lookup(Constants.CLIENT_LOCAL_TCF);
			System.out.println("MDB Test client: tcf = " + tcf.toString());

			// lookup the TopicConnectionFactory through its JNDI name
			qcf = (QueueConnectionFactory) ictx.lookup(Constants.CLIENT_LOCAL_QCF);
			System.out.println("MDB Test client: qcf = " + qcf.toString());

			// create or lookup the Topic through its JNDI name
			topic = Util.createTopicIfNotExists(
					Constants.LOCALHOST_CONNECTOR_MBEAN_NAME,
					Constants.JMX_URL, ictx,
					topicName, "");

			// create or lookup the Reply Queue through its JNDI name
			replyQueue = Util.createQueueIfNotExists(
					Constants.LOCALHOST_CONNECTOR_MBEAN_NAME,
					Constants.JMX_URL, ictx,
					Constants.T_MDB_REPLY_QUEUE_NAME, "");
			
		} catch (NamingException e) {
			
			System.out.println("Fail to create destination -> use JOnAS console" +
					"to create following destination and retry the test: ");
			
			System.exit(2);
		}
		
		System.out.println("Test Topic: " + topicName);
		System.out.println("Result Queue: " + Constants.T_MDB_REPLY_QUEUE_NAME);


		TopicConnection tc = null;
		TopicSession session = null;
		try {
			tc = tcf.createTopicConnection();
			System.out.println("MDB Test client: tc = " + tc.toString());
			session = tc.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
			tp = session.createPublisher(topic);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(2);
		}

		long beginSendTime = System.currentTimeMillis();

		try {
			System.out.println("Start to send " + nbmess
					+ " messages SEQUENTLY");
			TextMessage message;
			for (int i = 1; i <= nbmess; i++) {
				message = session.createTextMessage();
				message.setIntProperty(Constants.T_MDB_MSG_ID, i);
				message.setBooleanProperty(Constants.END_KEY, false);
				if (i == nbmess) {
					message.setBooleanProperty(Constants.END_KEY, true);
					initTestKey = session.toString();
					message.setStringProperty(Constants.TEST_KEY, initTestKey);
					message.setLongProperty(Constants.BEGIN_SEND_TIME_KEY,
							beginSendTime);
				}
				tp.publish(message);
			}
			session.close();
			tc.close();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(2);
		}

		try {
			QueueReceiver qReceiver;
			QueueConnection qc = qcf.createQueueConnection();
			System.out.println("JMS client: qc = " + qc.toString());
			QueueSession qSession = qc.createQueueSession(false,
					Session.AUTO_ACKNOWLEDGE);
			qc.start();

			qReceiver = qSession.createReceiver(replyQueue);

			System.out
					.println("Waiting for reply message from the test MDB with the timeout of "
							+ TIME_OUT + " miliseconds...");
			TextMessage msg = (TextMessage) qReceiver.receive(TIME_OUT);

			int numberMsgReceived = msg.getIntProperty(Constants.T_MDB_NUMBER_MSG_RECEIVED);
			long sendTime = msg.getLongProperty(Constants.SEND_TIME_KEY);
			String returnedTestKey = msg.getStringProperty(Constants.TEST_KEY);
			
			if (returnedTestKey.equals(initTestKey)) {
				System.out.println("Test success!");
			} else {
				System.out.println("Test fail!");
				
				
					System.out.println("Clear the result queue " +
							Constants.T_MDB_REPLY_QUEUE_NAME +
							" and retry the test");
			}
			
			System.out.println("Number of messages received is : " + numberMsgReceived);
			
			System.out.println("Time to send " + numberMsgReceived
					+ " messages from a client to an MDB is: " + sendTime);

			qSession.close();
			qc.close();
			System.in.read();
		} catch (Exception e) {
			System.out.println("Exception in message reception operations");
			System.out.println("Test fail! Can not receive the reply message");
			e.printStackTrace();
		}
	}
}