/**
 *
 * (c) Copyright BULL S.A.S. 2007.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Util.java 10249 2007-04-23 11:22:14Z waeselynck $
 * --------------------------------------------------------------------------
 */

package com.bull.jonasmqconnector.test.util;

import javax.jms.Queue;
import javax.jms.Topic;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.NamingException;

public class Util {
	/**
	 * Lookup for the topic, if not exists then create it with the MBeans
	 * 
	 * @param JMX_URL
	 *            the jmx url
	 * @param ictx
	 *            the context
	 * @param jndiName
	 *            the jndi name
	 * @param properties
	 *            the properties of the distanation
	 * @return the topic
	 */
	public static Topic createTopicIfNotExists(
			String connMBeanName,
			String jmxURL, Context ictx,
			String jndiName, String properties) {
		Topic topic = null;
		try {
			topic = (Topic) ictx.lookup(jndiName);
		} catch (NamingException e) {
			// e.printStackTrace();
			createTopicWithJMX(connMBeanName, jmxURL, jndiName, properties);
			
			try {
				topic = (Topic) ictx.lookup(jndiName);
			} catch (NamingException ne) {
				ne.printStackTrace();
			}
		}
		return topic;
	}
	
	/**
	 * Create a Topic with MBeans
	 * 
	 * @param JMX_URL
	 *            the jmx url
	 * @param jndiName
	 *            the jndi name
	 * @param properties
	 *            the properties of the distanation
	 * @return the topic
	 */
	public static void createTopicWithJMX(
			String connMBeanName,
			String jmxURL, String jndiName,
			String properties) {
		
		try {
			// properties = "Persistence=2"
			// javax.jms.DeliveryMode.PERSISTENT;
			JMXServiceURL connURL = new JMXServiceURL(jmxURL);
			JMXConnector connector = JMXConnectorFactory.newJMXConnector(
					connURL, null);
			connector.connect();
			MBeanServerConnection conn = connector.getMBeanServerConnection();
			ObjectName mbName = new ObjectName(connMBeanName);
			
			String[] signature = {"boolean", "java.lang.String",
			"java.lang.String" };
			if (properties == null)
				properties = "";
			Object[] params = { Boolean.TRUE , jndiName, properties };
			conn.invoke(mbName,
					"createDestination", params, signature)
					;
			
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * Lookup for the queue, if not exists then create it with the MBeans
	 * 
	 * @param JMX_URL
	 *            the jmx url
	 * @param ictx
	 *            the context
	 * @param jndiName
	 *            the jndi name
	 * @param properties
	 *            the properties of the distanation
	 * @return the queue
	 */
	public static Queue createQueueIfNotExists(
			String connMBeanName,
			String jmxURL, Context ictx,
			String jndiName, String properties) {
		Queue queue = null;
		try {
			queue = (Queue) ictx.lookup(jndiName);
		} catch (NamingException e) {
			createQueueWithJMX(connMBeanName, jmxURL, jndiName, properties);
			try {
				queue = (Queue) ictx.lookup(jndiName);
			} catch (NamingException ne) {
				ne.printStackTrace();
			}
		}
		return queue;
	}
	
	/**
	 * Create a Queue with MBeans
	 * 
	 * @param JMX_URL
	 *            the jmx url
	 * @param jndiName
	 *            the jndi name
	 * @param properties
	 *            the properties of the distanation
	 * @return the queue
	 */
	public static void createQueueWithJMX(
			String connMBeanName, 
			String jmxURL, 
			String jndiName,
			String properties) {
		try {
			JMXServiceURL connURL = new JMXServiceURL(jmxURL);
			JMXConnector connector = JMXConnectorFactory.newJMXConnector(
					connURL, null);
			connector.connect();
			MBeanServerConnection conn = connector.getMBeanServerConnection();
			ObjectName mbName = new ObjectName(connMBeanName);
			
			String[] signature = { "boolean", "java.lang.String",
			"java.lang.String" };
			if (properties == null)
				properties = "";
			Object[] params = { Boolean.FALSE, jndiName, properties };
			conn.invoke(mbName,"createDestination", params, signature);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}